// feld ist unser zweidimensionales Spielfeld, es besteht aus
// Zellen mit einer boolschen Variable, die man mit lebt? 
// (Ja/Nein) beschreiben könnte. 
// Vor der Benutzung muss der Speicher für die Zellen noch 
// mit "new" allokiert werden, wie für alle Felder
boolean feld[][];
boolean altesFeld[][]; // temporäres Spielfeld, siehe Prozedur
                       // "entwickleFelder()"

// groesseX und groesseY beschreiben die Größe des Spielfeldes,
// die horizontale und vertikale Anzahl Zellen, die wir auf unserem
// Zeichenfeld realisieren wollen. 
// Wandert eine Front zum linken Rand, so erscheint sie wieder am 
// rechten Rand (und umgekehrt), gleiches gilt für unten und oben.
// Man sagt, es handelt sich um ein torodiales Feld
//
int groesseX = 100;
int groesseY = 100;
int bildRate = 1;

// jetzt kommen einige Prozeduren, um das Spielfeld zu 
// initialisieren und auf einzelne Felder zuzugreifen

void initialisiereFelder(int x, int y) {
  groesseX = x;
  groesseY = y;
  feld = new boolean[groesseX][groesseY];
  altesFeld = new boolean[groesseX][groesseY];
  loescheFeld();
}

// alle Zellen werden auf "false" gesetzt

void loescheFeld() {
  for (int j=0; j<groesseY; j++) 
    for (int i=0; i<groesseX; i++) 
      feld[i][j] = false;
}

// alle Zellen werden zufällig auf true oder false gesetzt
// allerdings nicht gleichverteilt, sondern nur 30% werden 
// auf true gesetzt, denn dann entstehen schöne Fronten,
// verändern sie das einmal...

void setzeFeldZufaellig() {
  for (int j=0; j<groesseY; j++) 
    for (int i=0; i<groesseX; i++) 
      feld[i][j] = (random(0, 1)>0.7);
}

void setzeFeldMuster() {
  for (int j=0; j<groesseY; j++) 
    for (int i=0; i<groesseX; i++) 
      feld[i][j] = false;
  int x0 = groesseX/2;
  int y0 = groesseY/2;
  feld[x0   ][y0   ] = true;
  feld[x0+1 ][y0   ] = true;
  feld[x0+2 ][y0   ] = true;
  feld[x0+3 ][y0   ] = true;
  feld[x0+3 ][y0+1 ] = true;
  feld[x0+2 ][y0+2 ] = true;
  feld[x0+1 ][y0+3 ] = true;
  feld[x0   ][y0+3 ] = true;
  feld[x0-1 ][y0+3 ] = true;
  feld[x0-2 ][y0+3 ] = true;
  feld[x0-3 ][y0+2 ] = true;
  feld[x0-4 ][y0+1 ] = true;
  feld[x0-4 ][y0   ] = true;
  feld[x0-4 ][y0-1 ] = true;
  feld[x0-3 ][y0-2 ] = true;
  feld[x0-2 ][y0-3 ] = true;
  feld[x0-1 ][y0-3 ] = true;
  feld[x0   ][y0-3 ] = true;
  feld[x0   ][y0-2 ] = true;
  feld[x0   ][y0-1 ] = true;
 }

void setze(int x, int y, boolean v) {
  feld[x][y] = v;
}

boolean lebt(int x, int y) {
  return feld[x][y];
}


// "nachbarn" berechnet die Anzahl der Nachbarn einer Zelle
// die leben. Dies allerdings auf der Basis des temporären 
// Feldes "altesFeld" - dorthin müssen also vorher die Werte
// aus "feld" kopiert worden sein.
// die etwas seltsame Modifikation des x und y-Parameters dient
// dazu, auch am Rand ein korrektes Ergebnis zu erzielen. 
// Warum klappt das? 

int nachbarn(int x, int y) {
  int n = 0;
  if (x==0) x = groesseX;
  if (y==0) y = groesseY;
  
  if (altesFeld[(x+1)%groesseX][(y+1)%groesseY]) n++;
  if (altesFeld[(x+1)%groesseX][(y  )%groesseY]) n++;
  if (altesFeld[(x+1)%groesseX][(y-1)%groesseY]) n++;
  if (altesFeld[(x  )%groesseX][(y-1)%groesseY]) n++;
  if (altesFeld[(x-1)%groesseX][(y-1)%groesseY]) n++;
  if (altesFeld[(x-1)%groesseX][(y  )%groesseY]) n++;
  if (altesFeld[(x-1)%groesseX][(y+1)%groesseY]) n++;
  if (altesFeld[(x  )%groesseX][(y+1)%groesseY]) n++;   
  return n;
}

//-------------------------------------------------
// hier wird auf der Basis des existierenden Zustands des
// Spielfelds ein neuer Zustand berechnet. Dazu müssen wir
// alle aktuellen Werte aus "feld" in ein zweites Spielfeld
//  "altesFeld" kopieren, weil sonst die Anwendung der Regel 
// in einer Zelle die Nachbarn direkt beeinflussen würde. Wir
// möchten aber den neuen Zustand jeder Zelle aus ihren Nach-
// barzuständen zum aktuellen Zeitpunkt berechnen
//-------------------------------------------------

void entwickleFeld() {

  for (int y=0; y<groesseY; y++) 
    for (int x=0; x<groesseX; x++)
      altesFeld[x][y] = feld[x][y];

  for (int y=0; y<groesseY; y++) 
    for (int x=0; x<groesseX; x++) {
      // ermittle Anzahl Nachbarfelder die leben
      int count = nachbarn(x, y); 
      // Lebensregel
      if (!lebt(x, y) && (count == 3) ) 
         setze(x,y,true);
      if (lebt(x,y) && (count < 2 || count > 3) ) 
         setze(x,y,false);
      //if (!lebt(x, y) && ((count == 1) || (count == 3)|| (count == 5) || (count == 7))) 
      //   setze(x,y,true);
      //if (lebt(x,y) && ((count == 0) || (count == 2) || (count == 4) || (count == 6) || (count == 8))) 
      //   setze(x,y,false);
    }
} 

// hier wird das aktuelle Spielfeld gezeichnet

void zeichneFeld() {
  // noStroke();
  int rw = width / groesseX;
  int rh = height / groesseY;
  for (int y=0; y<groesseY; y++) 
    for (int x=0; x<groesseX; x++) {
      if (lebt(x, y)) fill(0,255,0); else fill(255,0,0);
      rect(x*rw, y*rh, rw, rh);
    }
}

/////////////////////////////////////////////////////////
// Standardprozeduren
/////////////////////////////////////////////////////////



void setup() {
  size(800, 800);
  background(0,0,0);
  initialisiereFelder(groesseX,groesseY);
  randomSeed(0);
  frameRate(bildRate++);
  setzeFeldZufaellig();
  // ein Muster als Anfangsonfiguration
  // setzeFeldMuster();
}

void draw() {
  frameRate(bildRate++);
  zeichneFeld();
  entwickleFeld();
}

void keyPressed() {
  if (key == 's') save("frame"+frameCount+".png");
  if (key == 'i') setzeFeldZufaellig();
}